function generate_TECGradientMap_TechTIDE(filename,outputTYPE,outputDIR)
%************************************************************************
%* This function generates a TEC Spatial Gradient map from an IMPC TEC map
%* This program has been designed to read IMPC TEC maps
%* (http://impc.dlr.de)
%* This code has been developed in the frame of the EU H2020 Project
%* TechTIDE www.tech-tide.eu
%*
%* input: filename (string): name of the json file with the TEC map, which
%*                           is going to be used for the generation of the 
%*                           TEC gradients
%*        outputDIR (string) : path and name of the directory where the
%*                             results are going to be stored
%* output: png image stored in outputDIR
%*         json data file containing the TEC gradient map
%*
%* author: C. Borries, German Aerospace Center (DLR)
%* date: 2018-12-12
%* terms of use: Public, Open Access, distributed under the terms of 
%*               Creative Commons Attribution License
%*               (http://creativecommons.org/licenses/by/4.0)
%************************************************************************

%***
%* The program requires to install the open source json_parser
%*  https://github.com/christianpanton/matlab-json/
%* addpath shall direct to the directory with the Matlab json_parser
%***
addpath('json_parser','Tools')

% check input parameters
if exist('filename')==1, 
    if ~(exist(filename)==2)
        disp('ERROR: The input filename does not exist!'), return , 
    end 
else disp('Please indicate your filename'),return,
end
if exist('outputTYPE')==1, 
    if ~(strcmp(lower(outputTYPE),'file')|| strcmp(lower(outputTYPE),'image')), disp('Please use correct outputTYPE (file or image)'),return;
    end
  else outputTYPE='image';end
if exist('outputDIR')==1
    % check if outputDIR is and existing directory
    if ~(exist(outputDIR)==7)
        disp('ERROR: The ouput directory does not exist!'), return     
    end
else
    outputDIR='';
end

tec_map = fromjson(fileread(filename)); %read json file

% extract the date information from the file
reqDate=datenum(tec_map.metadata.details.temporal_coverage.end_time,'yyyy-mm-ddTHH:MM:SS');

% read the TEC map data from the file
 lat_deg=zeros(1,length(tec_map.data.grid.features));
 lon_deg=zeros(1,length(tec_map.data.grid.features));
 vtec_assimilated_tecu=zeros(1,length(tec_map.data.grid.features));
 for x=1:length(tec_map.data.grid.features)
     lat_deg(x)=cell2mat(tec_map.data.grid.features{x}.geometry.coordinates(2));
     lon_deg(x)=cell2mat(tec_map.data.grid.features{x}.geometry.coordinates(1));
     vtec_assimilated_tecu(x)=tec_map.data.grid.features{x}.properties.vtec_assimilated_tecu;
 end
 
% generate latitude and longitude vectors
lat_map=unique(lat_deg);
lon_map=unique(lon_deg);

% calculate the distance between map grid cells
RE=6371; %km;
dist_lat=ones(1,length(lon_map)).*(lat_map(1)-lat_map(2))*pi*RE/180;
for xx=1:length(lat_map)
    dist_lon(xx)=distGC(lat_map(xx),0,lat_map(xx),lon_map(1)-lon_map(2))*pi*RE/180;
end

% generate a TEC 2D array with map character and corresponding 2D latitude
% and longitude distances 
map=reshape(vtec_assimilated_tecu,length(lon_map),length(lat_map));
[dist_lon2,dist_lat2]=meshgrid(dist_lon,dist_lat);

% calculate TEC gradients in latitude and longitude and absolute
grad_lat(:,:)=[zeros(1,length(lat_map)); (map(1:end-1,:)-map(2:end,:))]./dist_lat2; % [TECU/km]
grad_lon(:,:)=[(map(:,2:end)-map(:,1:end-1)), zeros(size(lon_map'))]./dist_lon2;
grad_lat=grad_lat.*160; % mm/km on L1 (1TECU=16cm)
grad_lon=grad_lon.*160; % mm/km on L1 (1TECU=16cm)
grad_tot=sqrt(grad_lat.^2+grad_lon.^2); % mm/km on L1

if strcmp(lower(outputTYPE),'file')
    % save output data file
    outputfilename=fullfile(outputDIR,['TECGradientMap_' datestr(reqDate,'yyyy-mm-ddTHH-MM-SS') '.json']);
    TECgradients.metadata.description='Changes of the Total Electron Content (TEC) with horizontal distance. TEC gradients are calculated from TEC maps.';
    TECgradients.metadata.temporal_coverage.start_time=tec_map.metadata.details.temporal_coverage.start_time;
    TECgradients.metadata.temporal_coverage.end_time=tec_map.metadata.details.temporal_coverage.end_time;
    TECgradients.metadata.data_name='European Total Electron Content (TEC) Gradients';
    TECgradients.metadata.data_unit='mm/km';
    TECgradients.metadata.created=datestr(now,'yyyy-mm-ddTHH:MM:SS');
    TECgradients.data.TEC_grad_lat=grad_lat;
    TECgradients.data.TEC_grad_lon=grad_lon;
    TECgradients.data.TEC_grad_tot=grad_tot;
    TECgradients.data.latitude=lat_map;
    TECgradients.data.longitude=lon_map;
    TECgradients.data.time=datestr(reqDate,'yyyy-mm-dd HH:MM');
    json_str=tojson(TECgradients);
    fid=fopen(outputfilename,'w');
    fprintf(fid,'%s',json_str);
    fclose(fid);
    disp(['Output: ' outputfilename])
elseif strcmp(lower(outputTYPE),'image')
    % Plot the TEC and TEC Gradient maps
    clf

    subplot(2,1,1)
    colormap('jet')
    imagesc(lon_map,lat_map,map'),caxis([0 50]),h_cb=colorbar;
    load coast;
    geoshow(lat,long,'color','k')
    set(gca,'ydir','normal')
    ylabel('Latitude [N]')
    xlabel('Longitude [E]')
    title(['TEC map, ' datestr(reqDate,'yyyy-mm-dd HH:MM') ' UT'])
    title(h_cb,'[TECU]')

    subplot(2,1,2)
    imagesc(lon_map,lat_map,grad_tot');caxis([0 6]),
    hold on
    quiver(lon_map,lat_map,grad_lat',grad_lon');
    h_cb=colorbar;
    geoshow(lat,long,'color','k')
    set(gca,'ydir','normal')
    ylabel('Latitude [N]')
    xlabel('Longitude [E]')
    title(['TEC gradient map, ' datestr(reqDate,'yyyy-mm-dd HH:MM') ' UT'])
    title(h_cb,'[mm/km]')

    % save image file
    outputfilename=fullfile(outputDIR,['TECGradientMap_' datestr(reqDate,'yyyy-mm-ddTHH-MM-SS') '.png']);
    set(gcf,'Paperposition',[0 0 15 15])
    saveas(gcf,outputfilename)
    disp(['Output: ' outputfilename])
    close all
    
end
end